/*
**  ADDT
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  {tw,Sz}=addt(y,X,w,la);
**
**                                To create a t-test for the added
**                                variable w
**
*/

/*
** ADDT
**
** PURPOSE:  to produce the t test for additional expl. variable
**
** FORMAT:  {tw,Sz}=addt(y,X,w,la);
**
** INPUT:     y  = n x 1 rensponse variable
**            X  = n x p data matrix; n observations
**                 and p explanatory variables
**             w = additional explanatory varible whose t test
**                 must be computed
**            la = scalar which specifies for which transformation
**                 you want to compute the tstat for the additional
**                 variable
**
** OUTPUT:     tw = scalar the t test for the added variable
**             Sz = new estimate of s^2
**
** BACKGROUND: expression for t statistic uses equation (2.30)
**             see Atkinson Riani (2000)
*/


 proc (2)=addt(y,X,w,la);

   local n,p,G,A,z,Tl,Sz,r,zAw,wAw;

        n=rows(X);
        p=cols(X);
        G=exp(meanc(ln(y)));  @ Geometric mean of th observations @
        A=eye(n)-X*invpd(X'X)*X';  @ Hat Matrix @
           if     la==0;
                      z=G*ln(y);
              else;
                      z=(y^la-1)/(la*G^(la-1));
           endif;
          Tl=miss(0,0);

            r=z'A*z;
            zAw=z'A*w;   wAw=w'A*w;
            
            if abs(wAw) <1e-12;
              Sz=miss(0,0);
              Tl=miss(0,0);
              goto fin;
            else;
             Sz=sqrt(r-zAw^2/wAw); @ See Atkinson (1985) p. 98 @
            endif;

              if ABS(real(Sz)) < 0.0001;
              else;
                   Tl= zAw*sqrt(n-p-1)/(Sz*sqrt(wAw));
              endif;
        fin:

  retp(Tl,Sz^2/(n-p-1));
endp;

