/*
**  ADDVAR
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  addvar(y,X,la);
**
**                                To create an added variable plot
**
**
*/

/*
** ADDVAR
**
** PURPOSE:  to produce an added variable plot (the added varibale is the constructed variable
**           which is created automatically from scalar la. The most common values 
**           of la are (-1,-0.5,0,0.5,1)  
**
** FORMAT:   addvar(y,X,la);
**
** INPUT:     y  = n x 1 rensponse variable
**            X  = n x p data matrix; n observations
**                 and p explanatory variables
**            la = scalar which specifies for which transformation
**                 you want to watch the added variable plot
**
**
** OUTPUT:     The added variable plot is produced on the screen
**
** BACKGROUND: On the y-axis there are the residuals from regression
**             On the x-axis there are the residuals from constructed
**              variable (w)
**             A=I -X* inv(X'X)* X'
**             yaxis A*y
**             xaxis A*w
*/


 proc (0)=addvar(y,X,la);

   local n,p,G,A,z,w,xaxis,yaxis,Ux,Uy,Lx,Ly,t;

        n=rows(X);
        p=cols(X);
        G=exp(meanc(ln(y)));  @ Geometric mean of th observations @
        A=eye(n)-X*invpd(X'X)*X';  @ Hat Matrix @
       @ w= Vector which contains the constructed variable @
           if     la==0;
                      z=G*ln(y);
                      w=G*ln(y).*(ln(y)/2-ln(G));
           else;
                      z=(y^la-1)/(la*G^(la-1));
                      w=(y^la.*ln(y)-(y^la-1)*(1/la+ln(G))
                         ) /(la*G^(la-1));
           endif;

         /* Added variable plot for constructed variable w_p(1) */
             @ xaxis = residuals from constructed variable = A*w_p(l)
                                               =w^_p*(l) @
             @ yaxis = residuals from z = A*z = r @
              xaxis=A*w;
              yaxis=A*z;
              _plctrl=-1;
        xy(xaxis,yaxis);
  retp;
endp;

