/*
**  ADDVAR
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  addvarw(y,X,w);
**
**                                To create an added variable plot
**
**
*/

/*
** ADDVAR
**
** PURPOSE:  to produce an added variable plot
**
** FORMAT:   addvar(y,X,w);
**
** INPUT:     y  = n x 1 rensponse variable
**            X  = n x p data matrix; n observations
**                 and p explanatory variables
**            w = n x 1 vector
**                which specifies the variable which is
**                 added
**
**
** OUTPUT:     The added variable plot is produced on the screen
**
** BACKGROUND: On the y-axis there are the residuals from regression
**             On the x-axis there are the residuals from constructed
**              variable (w)
**             A=I -X* inv(X'X)* X'
**             yaxis A*y
**             xaxis A*w
*/


 proc (0)=addvarw(y,X,w);

   local n,A,xaxis,yaxis;

        n=rows(X);
        A=eye(n)-X*invpd(X'X)*X';  @ Hat Matrix @
       @ w= Vector which contains the constructed variable @

         /* Added variable plot for constructed variable w_p(1) */
             @ xaxis = residuals from constructed variable = A*w_p(l)
                                               =w^_p*(l) @
             @ yaxis = residuals from z = A*y = r @
              xaxis=A*w;
              yaxis=A*y;
              _plctrl=-1;

  @  xglobal=xaxis~yaxis;   @
         output file=C:\gauss\ttest\temp.tmp reset;
          sortc(seqa(1,1,n)~xaxis~yaxis,2);
         output file=C:\gauss\ttest\temp.tmp off;

        xy(xaxis,yaxis);
  retp;
endp;

