/*
** ANDREWS.G
** written by MARCO RIANI
**
** FORMAT                      PURPOSE                     Line
** =======================================================================
** {a,B}=andrews(X)          Creates Andrews curves          29
**                           (see Andrews, 1972;
**                           Biometrics, 28 p. 125-136)
*/

/*
**
** ANDREWS
**
** PURPOSE:  represents each row of a data matrix with
**           a trigonometric curve defined in the
**           interval (-\pi, +\pi).  (creates a plot)
**
** FORMAT:   {a,B}=andrews(X)
**
** INPUT:     X= n x p data matrix (n observations on p variables)
**
** OUTPUT:    a= 63 x 1 vector. Sequence of 63 terms from (-\pi +\pi)
**            B= 63 x n matrix. Each column represents
**                   a trigonometric curve.
**                   Ex: 5th col. contains the curve refered to the
**                       5th unit (5th row of input matrix X)
*/

proc  (2)=andrews(X);
                local ft,cc,tt,t,i,j,e,ccc,a,n,p;

CC=ZEROS(100,1);
tt=1;
n=rows(X);
p=cols(X);
ft=zeros(n,100);
  if  0.5*p-round(0.5*p) ne 0; @ checks if the number of cols.
                                 of data set is even or odd @
     t=-pi;  do until t>=pi;
              i=1; do until i==n+1;
                ft[i,tt]=x[i,1]/sqrt(2);
                   j=1; do while j<=p-2;
                                ft[i,tt]=ft[i,tt]+x[i,j+1]*sin(
                                  ((j+1)/2)*t)+x[i,j+2]*cos(((j+1)/2)*t);
                      j=j+2;
                   endo;
                 i=i+1;
              endo;
            tt=tt+1;
         CC[tt]=CC[tt]+tt;
       t=t+0.1;
    endo;
 else;
   t=-pi;  do until t>=pi;
       i=1; do until i==n+1;
           ft[i,tt]=x[i,1]/sqrt(2);
                 j=1; do while j<=p-2;
                   ft[i,tt]=ft[i,tt]+x[i,j+1]*sin(
                    ((j+1)/2)*t)+x[i,j+2]*cos(((j+1)/2)*t);
                 j=j+2;
               endo;
            ft[i,tt]=ft[i,tt]+x[i,j+1]*sin(((j+1)/2)*t);
         i=i+1;
        endo;
            tt=tt+1;
           CC[tt]=CC[tt]+tt;
        t=t+0.1;
      endo;
 endif;
    e=(cc.== 0);
    ccc=delif(cc,e);
    ft=ft[.,1:rows(ccc)];
    ft=ft';
    a=seqa(-pi,0.1,rows(ccc));
    xy(a,ft[.,1:cols(ft)]);
       retp(a,ft[.,1:cols(ft)]);
    endp;

