/*
**  ELMSR
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**
**  {SA} =elmsr(n,k1)             Enumarate all subsamples
**                                using sampling with replacement
**                                and ordered samples
*/


/*
**
** ELMSR
**
** PURPOSE:  to enumarate all possible subsets of
**           dimension k which can be extracted from
**           n elements. Sampling is with replacement
**           The total number of subsets is equal to C_{n+k-1,k}
**           (n+k-1 choose k)
**
** FORMAT:
**
** {SA} = elmsr(n,k1)
**
** INPUT:      n  = scalar number of elements (statistical units)
**             k1 = scalar dimension of the subsets
**
** OUTPUT:     sa = matrix with (n+k1-1)!/(k1!*(n-1)!) rows
**                  and k1 columns
**                  In each row there is a possible subset
**
**
*/

 proc (1)=elmsr(n,k1);

local i,ns,sa,k,b,j,a,c,d,e,f;
if  (k1-round(k1)) ne 0;
  errorlog "ERROR: dimension of subsamples must be an integer";
   end;
     elseif k1 > 7;
  errorlog "SORRY not implemented for subsamples greater than 7";
   end;
     else;
     endif;

   i=n+k1-2; ns=n+k1-1; do until i==n-1;
       ns=ns*i;
         i=i-1;
        endo;
     ns=ns/k1!;
  @ NS;  WAIT;   @
sa=zeros(ns,k1);
k=1;
  if k1==1;
    sa=seqa(1,1,ns);
  elseif k1==2;
        @ possible groups of two elements @
         b=1; do until b==n+1;
            j=b; do until j==n+1;
                sa[k,.]=b~j;
           k=k+1;
        j=j+1;
      endo;
    b=b+1;
   endo;
  elseif k1==3;
@ posible groups of three elements @
a=1;  do until a==n+1;
b=a; do until b==n+1;
j=b; do until j==n+1;
  sa[k,.]=a~b~j;
           k=k+1;
      j=j+1;
endo;
    b=b+1;
endo;
   a=a+1;
endo;
              elseif k1==4;

@ possible groups of four elements @
c=1; do until c==n+1;
a=c;  do until a==n+1;
b=a; do until b==n+1;
j=b; do until j==n+1;
  sa[k,.]=c~a~b~j;
           k=k+1;
                        j=j+1;
                    endo;
                b=b+1;
            endo;
        a=a+1;
     endo;
   c=c+1;
 endo;

              elseif k1==5;
@ possible groups of five elements @
d=1; do until d==n+1;
c=d; do until c==n+1;
a=c;  do until a==n+1;
b=a; do until b==n+1;
j=b; do until j==n+1;
  sa[k,.]=d~c~a~b~j;
           k=k+1;
                        j=j+1;
                    endo;
                b=b+1;
             endo;
          a=a+1;
       endo;
     c=c+1;
   endo;
 d=d+1;
endo;

              elseif k1==6;
@ possible groups of six elements @
e=1; do until e==n+1;
d=e; do until d==n+1;
c=d; do until c==n+1;
a=c;  do until a==n+1;
b=a; do until b==n+1;
j=b; do until j==n+1;
  sa[k,.]=e~d~c~a~b~j;
                                        k=k+1;
                                   j=j+1;
                             endo;
                         b=b+1;
                       endo;
                   a=a+1;
                  endo;
              c=c+1;
            endo;
         d=d+1;
        endo;
     e=e+1;
endo;

              elseif k1==7;
@ possible groups of seven elements @
f=1; do until f==n+1;
e=f; do until e==n+1;
d=e; do until d==n+1;
c=d; do until c==n+1;
a=c;  do until a==n+1;
b=a; do until b==n+1;
j=b; do until j==n+1;
  sa[k,.]=f~e~d~c~a~b~j;
                                        k=k+1;
                                   j=j+1;
                             endo;
                         b=b+1;
                       endo;
                   a=a+1;
                  endo;
              c=c+1;
            endo;
         d=d+1;
        endo;
     e=e+1;
  endo;
 f=f+1;
endo;
           else;

       endif;
           retp(sa);
   endp;

