/*
**  FWDOLSmdr
**  written by MARCO RIANI
**
**  FORMAT                                    PURPOSE               Line
** =======================================================================
**
**  {BB,s2,dres}=fwdolsmdr(X,y,la,Rou,sh,start);
**
**                                            This a simplified version of
**                                            fwdols.s. It just reports b coefficients
**                                            s^2 and minimum deletion residual
**
**
*/


/*
**
** FWDOLS
**
** PURPOSE:  Simplified version of procedure fwdols.g 
**
**
**
** FORMAT:
**
**  {BB,s2,dres}=fwdolsmdr(X,y,la,Rou,sh,start);
**
** INPUT:      X  = n x p matrix of explanatory variables
**             y  = n x 1 vector which contains the response variable
**             la = scalar which defines the
**                  transformation parameter \lambda
**                  for the response vector
**            Rou = Rou is a scalar or vector which specifies how to start
**                  the fwd search
**                  If rows(Rou)=1 the best subset is extracted according to
**                  lms else 
**                  if rows(Rou)> 1 we use LTS
**             sh = scalar which specifies the ordering
**                  of the residuals in the fwd search
**                  if sh=1      r_i^2= e_i^2             if i \in  BSB
**                               r_i^2= e_(i)^2/(1+h_(i)) if i \in  NCL
**                  if sh=2      r_i^2= e_i^2/(1-h_i)      if i \in  BSB
**                               r_i^2= e_(i)^2/(1+h_(i))  if i \in  NCL
**                  else         r_i^2= e_i^2     if i \in  BSB
**                               r_i^2= e_(i)^2   if i \in  NCL
**            start = specifies the starting point for fwd search
**                    fwd search starts with an initial subset
**                    size equal to p+start
**
** OUTPUT:  BB = n-p+1 x p matrix
**               beta coefficients in each step of the fwd search
**         S2  = n-p+1 x 3 matrix which
**               1st col = fwd search index
**               2nd col = var. of residuals
**                         = \sum e_i^2 /(m-p)
**               3rd col = r^2
**        DRES = (n-p-1) x 2 matrix which will contain min deletion
**               residual among nobsb  r_i^*  eq. 3.1.1
**               Remark: minimum deletion residual is stored with sign
**
**  REMARK1    this program does not add a column of 1 in matrix X to
**             create the intercept.
**
*/

proc   (3) =  fwdolsmdr(X,y,la,Rou,sh,start);
   local  as,ku,nor,ram,s2,sco,dres,
          cc,rres,un,tout,covr,fr,lev,d,
          r,n,p,i,j,tp,mIn,BB,TT,PVT,LIK,RST,Coo,
          diff,mt,seq,HH,IH,mE,lm,b1,r2,rs,jk,unit,bs,me1,mm,
          bsb,mX,ncl,vY,vYt,Yt,EE,mmX,oldmmX,mAm,b,sres,rsq,NO,
          RA,sc,Hi,sor,bib,LI,e,k,ord,oldbsb,pr,se,Unitout,posress,
          posresU,selDres;
  n=rows(X);
  p=cols(X);
  tp=start;
d=0; @ d=0 implies not a time series @
pr=0; @ p=0 implies we do not print the output from procedure regressi @
seq=seqa(1,1,n);
R=seq~zeros(n,1); @ The second column of matrix R will contain the
                    OLS residuals at each step of the forward search   @


     {mE,lm,BS}=lms(X,y,la,Rou);
 
  @ Matrix BB will contain the beta coeff. in each step of the fwd search @
  BB=seqa(p,1,n-p+1)~zeros(n-p+1,p);

@ S2=(n-p+1) x 3  matrix which will contain
    1st col = fwd search index
    2nd col = S2= \sum e_i^2 / (n-p)
    3rd col = R^2 @
  S2=seqa(p,1,n-p+1)~zeros(n-p+1,2);

  @ DRES= (n-p-1) x 2 matrix which will contain min deletion residual
    among nobsb  r_i^*  eq. 3.1.1 @
  DRES=seqa(p+1,1,n-p-1)~miss(zeros(n-p-1,1),0);

   jk=0; @ this is a dummy index linked to the columns of matrix mIn @
   unit=0;



            mE1=sortc(seq~abs(mE[.,1]),2);
                               start:
                 mm=p+tp;
                  @ bsb contains the indexes of the units forming BSB @
                      if Rou==1;
                         Bsb=BS;
                      else;
                         bsb=ME1[1:mm,1];
                      endif;
                       mX=X[bsb,.]; @ initial basic subset for X @
              
                   if rank(mX)<p;
                     "rank less than p";; wait;
                      tp=tp+1;
                      goto start;
                   else;
                   endif;
                   ncl=setdif(seq,bsb,1);
                  vY=y[bsb];   @ inital basic subset for y @
                  @ Transform all the units of the response var. @
                     if la==0;
                         vYt=ln(vY);
                         Yt=ln(Y);
                     else;
                         vYt=vY^(la);      @ transform the subset @
                         Yt=(Y^la);        @ transform all the units @
                     endif;

         mm=p+tp; do until mm==n+1;
         @ "mm";; mm;  @
                    mAm=mX'mX;
                    mmX=invpd(mAm);

              @ ols on basic subset using transf. response @
                  {B,sres,Rsq,NO,RA,SC}=regressi(vYt,mX,d,pr);

                  BB[mm-p+1,2:p+1]=(B[.,1]'); @ store beta coeff. @

             @store res. sum of squares/(mm-k)   @
                      S2[mm-p+1,2]=Rsq[1,2];
                      S2[mm-p+1,3]=Rsq[1,1];    @ Store R^2 @

            @ res. for the overall sample using transf. response
              and beta based on units forming BSB @
            e=Yt-X*B[.,1];

            r[.,2]=e.^2;    @ r_i =e_i^2 @

         if mm<n;

                   @ initialize ord @
                   ord=10000~0;
            if mm>p+tp;
                 k=1; do until k==rows(ncl)+1;
                   @ Take minimum deletion residual among noBSB @
                     hi=X[ncl[k],.]*mmX*(X[ncl[k],.]');
                     ord=ord|(r[ncl[k],2]/(1+hi)~e[ncl[k]]);
                    k=k+1;
                  endo;


                  @ Matrix Dres will contain min. among deletion residuals @
                     if S2[mm-p+1,2]>0;
                       selDres=sortc(ord,1);
                     
                      if selDres[1,2]>=0;
                       Dres[mm-p,2]=sqrt(seldres[1,1]/S2[mm-p+1,2]);
                      else;
                       Dres[mm-p,2]=-sqrt(seldres[1,1]/S2[mm-p+1,2]);
                      endif;
                        
                     else;
                     endif;
                 else;
                 endif;


              oldbsb=bsb;
                        
              @ order the r_i and include the smallest among the units
                forming the group of potential outliers @
                   ord=sortc(R,2);
                   @ bsb= units forming the new basic subset @
                   bsb=ord[1:(mm+1),1];
                   mX=X[bsb[.,1],.];  @ subset of X @
                   vY=Y[bsb[.,1]];    @ subset of untransformed Y @
                   vYt=Yt[bsb[.,1]];  @ subset of transformed Y @
                     if mm+1 < n;
                        @ ncl= units forming the new noclean @
                        ncl=ord[mm+2:n,1];
                     else;
                     endif;

          else;
         endif;

                mm=mm+1;
          endo;


 retp(BB,S2,dres);
endp;

