/*
**  GLMDEL
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  {Dres}=glmdel(X,y,q,fr,mint,tol,h)  To compute deletion
**                                        diagnostics for
**                                        glm
**
*/


/*
**
** GLMDEL
**
** PURPOSE:  to compute deletion diagnostics for
**           GLM
**
**
**
** FORMAT:   {Dres}=glmdel(X,y,q,fr,mint,tol,h);
**
** INPUT:     X    = N x p matrix
**            h    = scalar dummy variable
**                   if d <> 1 no plot is produced
**                   if d=1 a plot of C is calculated
**                   (potential outlier is a unit which
**                    shows a sharp drop in the plot)
** OUTPUT:  Dres   = N x 4 matrix
**                   the first column contains the indexes of the
**                   units, the second column contains deviance
**                   deletion residuals for each unit
**                   3rd col =reduction in deviance when unit i is
**                   omitted
**                   4th col = sign sqaure root of reduction in
**                   deviance
**
** REMARK:   This method is very good when only one outlier is present
**           in the data, but it can suffer masking and swamping
**           problems when there are several outliers.
**           In order to overcome these problems use
**           procedure FWDGLM
**
*/


proc (1)=glmdel(X,y,q,fr,mint,tol,h);

  local i,n,Xd,yd,frd,Dres,B,RS,Hi,WW,R,cnt,seq,bsb,Dev,Rsall;

  n=rows(X);
  Xd=X;
  yd=y;
  frd=fr;
  seq=seqa(1,1,n);
  Dres=seq~zeros(n,3);
          @ Store the deviance using all the  observations @
          {B,RSall,Hi,WW,R,cnt}=glm(X,y,q,fr,mint,tol);
          Dev=R[1,2];

 i=1;    do until i==n+1;
  bsb=setdif(seq,i,1);
  X=Xd[bsb,.];
  y=yd[bsb];
  fr=frd[bsb];

          {B,RS,Hi,WW,R,cnt}=glm(X,y,q,fr,mint,tol);
                      if cnt==1;
                         "Attention convergence";
                         "NOT obtained when excluding unit";; i;
                            wait;
                      else;
                      endif;

               @   COMPUTE RESIDUAL FOR i-th unit
                  and beta based on units forming BSB @
             {RS}=predglm(Xd[i,.],yd[i],q,frd[i],B[.,1]);
                 @ Deviance residual        Deviance using (n-1) obs. @
             Dres[i,2:3]=Rs[1,3]        ~         R[1,2];
                 "deleting unit";; i;
                 i=i+1;
        endo;
         Dres[.,3]=Dev-Dres[.,3];
      @ In the fourth col. we compute
        the sign square root of modification in deviance @
      i=1; do until i==n+1;
         if Rsall[i,3]>=0;
            Dres[i,4]=sqrt(Dres[i,3]);
         else;
            Dres[i,4]= -sqrt(Dres[i,3]);
         endif;
        i=i+1;
       endo;

    if h==1;
       _pdate="";
     title("Deletion diagnostics based on deviance rsiduals");
       xy(Dres[.,1],Dres[.,2]);
    else;
    endif;

      retp(Dres);
    endp;

