/*
**   HULL.G
**   written by ALDO CORBELLINI & MARCO RIANI
**
**  FORMAT                         PURPOSE                        	Line
** =======================================================================
** {H50,zleft}=hull(v,alpha)       Computes convex hull peeling   	46
**                                 up to find the (1-\alpha)% hull.
**                                 (1-\alpha)% hull is defined as the
**                                 biggest convex polygon containing
**                                 not more than (1-\alpha)% of the
**                                 observations
**
*/

/*
**
** HULL
**
** PURPOSE:  To perform convex hull peeling
**
** FORMAT:   {H50,zleft}=HULL(v,alpha)
**
** INPUT:     v = n x 3 matrix of data
**                  1st col = x coordinates
**                  2nd col = y coordinates
**                  3rd col = row index of the units
**        alpha = scalar between 0 and 1
**                if alpha = 1 all the hulls are computed
**                if alpha = 0 only the first hull is computed
**                if alpha = 0.6 hulls are computed up to the
**                first which contains less or equal than 40%
**                of the observations
**
** OUTPUT:  H50 = l x 3 matrix which contains the coordinates
**                of the (1-alpha)% hull
**                1st col = x coordinates
**                2nd col = y coordinates
**                3rd col = indexes of the units forming
**                          the (1-alpha)% hull
**        zleft = (n-l) x 3 matrix which contains the coordinates
**                of the points which are inside the (1-alpha)% hull
**                1st col = x coordinates
**                2nd col = y coordiantes
**                3rd col = indexes of the units which lie
**                          inside the (1-alpha)% hull
**
** See also: SPLINE
**
*/


proc (2)=hull(v,alpha);
  local zeta,z,lim,Hu,Hul,i,ii,zleft,rv,k1,rz;

    Hul=miss(zeros(60,2),0);
    rz=-1;
        lim=rows(v)*(1-alpha);
    i=0;   do until  i==20000;
              {zeta,z}=wrap(v);
              if i<=-6;
                  "zeta";
                  zeta[1:z,.];
                  wait;
               else;
               endif;
              @ z= number of points forming the hull
                zeta= rows(v) x 3 matrix
                      in the first z rows it contains
                      the coordinates of the points forming the hull
                      in the other rows it contains the
                      coordinates of the remaining points  @
                      @  rows(zeta); z; wait;@

     @  Hul= Matrix which contains the coord. of the points forming
        the vectices of the hulls @
          if rows(zeta[1:z,.])+1 > rz;
              rz=rows(zeta[1:z,.])+1;
           else;
          endif;

                ii=i;
               Hu=zeta[1:z,1:2]|zeta[1,1:2];
               Hul=Hul~miss(zeros(60,2),0);
              @ ii; cols(Hul); rows(HU); wait; @
               Hul[1:rows(Hu),2*ii+1:2*(ii+1)]=Hu;
                           @   "lim";;  lim;   @

                if rows(zeta)-1 <=lim or rows(zeta)-z<=2;
                      @  "jjjjj"; wait;  @
                    i=19999;
                else;
                    v=zeta[z+1:rows(zeta),.];
                endif;


        i=i+1;
     endo;     @     "jjjj";
           zeta; z; wait;    @
          Hul=Hul[1:rz,1:cols(Hul)-2];
          @ Hul[1:9,.]; wait;   @
     @ zleft= Matrix which contains the coordinates of the points lying
              inside the (1-\alpha)%-hull @
        if rows(zeta)>z;
           zleft=zeta[z+1:rows(zeta),.];
        else;
          zleft=miss(0,0);
        endif;
     retp(Hul,zleft);
   endp;

proc (1)=theta(a,b);
  local dx,dy,ax,ay,t;
    dx = b[1,1] - a[1,1];
    ax=abs(dx);
    dy = b[1,2] - a[1,2];
    ay=abs(dy);
        if (ax+ay)==0;
            t=0;
        else;
            t=dy/(ax+ay);
        endif;
        if dx < 0;
            t=2-t;
        elseif dy < 0;
            t=4+t;
        endif;
    retp(t*90);
endp;


proc (2) = wrap(d);
  local i,j,min,m,n,thi,vi,ti,dista,ang,flag,slin,dtemp,sel,min1,tmp;
      @ min contains the index of the row of matrix d
        which has the smallest y coordinate @

      @ An extra column is added to matrix d @
      d=d~seqa(1,1,rows(d));

                 @ preliminary checking for collinear points @
                    dtemp=selif(d,d[.,2].==minc(d[.,2]));
                    dtemp=sortc(dtemp,1);
                   @ The following loop remove duplicates
                     in matrix dtemp @
                   if rows(dtemp)>1;
                       sel=1;
                    i=1; do until i==rows(dtemp);
                       if dtemp[i,1:2]==dtemp[i+1,1:2];
                        else;
                          sel=sel|i+1;
                        endif;
                       i=i+1;
                      endo;
                     dtemp=dtemp[sel,.];
                   else;
                   endif;
                    if rows(dtemp)>1;
                       sel=sortind(dtemp[.,4]);
                     i=1; do until i==rows(dtemp)+1;
                          min=dtemp[sel[i],4];
                          ti=d[i,.];
                          d[i,.]=d[min,.];
                          d[min,.]=ti;
                        i=i+1;
                      endo;
                      @ This additional sorting is necessary
                        beacuse the collinear points may not
                        be ordered anymore @
                      tmp=sortc(dtemp[1:rows(dtemp),.],1);
                      d[1:rows(dtemp),.]=tmp;

                    @  d[1:rows(dtemp),.]; wait;  @
                                     @  ;   tmp;  wait;  @
                    else;
                        min=sortind(d[.,2]);
                        min=min[1];
                    endif;

    d=d|zeros(1,4);
    n=rows(d);

   /*** Last row of matrix d contains the sentinel */
   if rows(dtemp)>1;
        min=rows(dtemp);
        d[n,.]=d[1,.];
   else;
       d[n,.]=d[min,.];
   endif;

    thi=0.0001;
         m=rows(dtemp);   do while m <= n;
            /* swap the current min point position with sentinel*/
            ti=d[m,.];
            d[m,.]=d[min,.];
            d[min,.]=ti;
            min=n;
            vi=thi; thi=360; dista=10000; flag=0;
                j=m+1; do while j <= n;
                    {ang}=theta(d[m,.],d[j,.]);
                          if  ang >= vi and  ang < thi;
                                    min=j;
                                    {thi}=theta(d[m,.],d[j,.]);
                          endif;
                 j=j+1;
                endo;
                j=m+1; do while j <= n;
                        if theta(d[m,.],d[j,.]) == thi;
                           if sqrt(sumc((d[m,1:2]'-d[j,1:2]')^2))<dista;
                               dista=sqrt(sumc((d[m,1:2]'-d[j,1:2]')^2));
                               min=j;
                            endif;
                        endif;
                         @ The following if is necessary because
                           the coordinates of the point
                           forming the sentinel may not be unique  @
                          if d[min,1:2]==d[rows(d),1:2] and min <n;
                            goto two;
                             else;
                           endif;
                j=j+1;
                endo;
      if min==n;
           goto two;
       endif;
     m=m+1;
   endo;
  two:
 retp(d[1:rows(d)-1,1:3],m);
endp;

