/*
**  INPUTBOX
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {quant,out}=inputbox(X);           To compute necessary
**                                    values in order to
**                                    create a vertical
**                                    boxplot
**
*/


/*
**
** INPUTBOX
**
** PURPOSE:  to compute necessary values in order to construct a boxplot
**
** FORMAT:   {quant,out}=inputbox(X);
**
** INPUT:     X     = N x 1 vector of data
**                    or N x 2 matrix 
**                      If this case the second columns contains the indexes
**                       of the units 
**
** OUTPUT:    a     = 5 x 1 vector,
**                    a[1]=lower truncation point,
**                    a[2]= first quartile (X025),
**                    a[3]= median,
**                    a[4]= third quartile (X075),
**                    a[5]= upper truncation point
**            out   = (k+1) x 2 matrix, which contain the eventual
**                    univariate outliers for vector X
**                    the first row is a zero vector of dimension 1 x 2
**                    1st col = labels
**                    2nd col = coordinates
**
** REMARK: the output of this program can be used in BOXPLOT.G
**
*/




proc (2)=inputbox(X);
local seq,x2,i,z,Xmed,X025,X075,Ts,Ti,Tsup,Tinf,di,cc,c,
      quant,cinf,out,sel,n,lab;
  @ creation and sorting of matrix x1 @


 n=rows(X);
 c=rows(x);

if cols(x)==1;
 seq=seqa(1,1,c);
else;
 seq=X[.,2];
 X=X[.,1];
endif;
 x=seq~x;


 x=sortc(x,2);
@ calculation of X025 @
c=rows(x)*0.25;
cc=round(c);
cinf=x[cc,2];
if (c-round(c))==0;
X025=x[c,2];
else;
X025=(x[cc,2]+x[cc+1,2])/2;
endif;         @  X025  is the first quartile @

@ calculation of median @
c=rows(x);
if (c/2-round(c-c/2))==0;
Xmed=(x[c/2,2]+x[c/2+1,2])/2;
else;
Xmed=x[(c+1)/2,2];
endif;         @  Xmed  is the median  of vector X @

@ calculation of third quartile @
c=rows(x)*0.75;
cc=round(c);
cinf=x[cc,2];
z=0;
if (c-round(c))==0;
X075=x[c,2];
else;
X075=(x[cc,2]+x[cc+1,2])/2;
endif;         @  X075  is the third quartile @

DI=X075-X025;

Tsup=X075+DI*1.5;   @ upper truncation point @
Tinf=X025-DI*1.5;   @ lower truncation point @

/*******
  if Tsup>x[rows(x),2];
      Tsup=x[rows(x),2];
    else;
  endif;

  if Tinf<x[1,2];
       Tinf=x[1,2];
   else;
  endif;

**********/
              @   Tsup; wait;   @
                     if Tsup>x[n,2];
                              Tsup=x[n,2];
                     else;
                              @ Find upper truncation point @
                              i=1; do until i==n+1;
                               if x[n-i,2]>Tsup;
                               else;
                                Tsup=x[n-i,2];
                                i=n;
                               endif;
                                i=i+1;
                               endo;
                     endif;

                     if Tinf<x[1,2];
                              Tinf=x[1,2];
                     else;

                              @ Find lower truncation point @
                              i=1; do until i==n+1;
                               if x[i,2]<Tinf;
                               else;
                                Tinf=x[i,2];
                                i=n;
                               endif;
                                i=i+1;
                               endo;
                     endif;

/** the following lines check for outliers **/
out=0~0;
i=1; do until i==rows(x)+1;
        if x[i,2]<Tinf or x[i,2]>Tsup;
           out=out|(x[i,1]~x[i,2]);
        else;
        endif;
i=i+1;
    endo;

@ MODIFIED CHECK CHECK sel contains the rows associated to the good units
  (excluding the outliers) 
  sel=setdif(lab[.,1],out[.,1],1);
   x=x[sel,.]; @
  
quant=Tinf|x025|Xmed|X075|Tsup;
@ quant; wait;   @
 retp(quant,out);
  endp;

