/*
**  LRDEL (deletion diagnostics for likelihood ratio )
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {lrde}=lrdel(X,la,h)
**
**                                To compute del. diagnostics
**                                for likelihodd ratio
**                                The purpose is to test the null
**                                hypothesis that the transformation
**                                parameters are as those specified in
**                                vector la.
**
**
*/

/*
** LRDEL
**
** PURPOSE:  To compute del. diagnostics for lik. ratio
**
** FORMAT:   {lrde}=lrdel(X,la,h)
**
** INPUT:     X = n x p data matrix; n observations
**                and p variables
**           la = p x 1 vector specifies how the p variables in matrix
**                X should be transformed.
**                For example la={0,0.5} suggests a log transformation
**                for the first variable and s sqrt tr. for the second.
**            h = scalar. If h=1 a plot which contains the values
**                of the del. likelihood ratio appears on the screen
**
** OUTPUT: lrde =  n x 2 matrix
**                 1st col. index from 1 to n
**                 2nd col. lr deletion diagnostic test
**
** SEE ALSO: LRADDEL
*/


 proc (1)=lrdel(X,la,h);

      local seq,seq1,i,n,nla,pos,pot,likmax,lr,lrde,plik,
            lamax,bsb,In,Un,ch;

  n=rows(X); nla=rows(la);
  seq=seqa(1,1,n);
  seq1=seq[1:n-1];
 lrde=seq~miss(zeros(n,1),0);
 ch=seqa(1,1,nla)~la;
 @ We use procedure flr for computing
   maximum likelihood estimates of transformation
   parameters using all the observations @
    @ row vector plik contains mle of transformation parameters @

    @ We use procedure fwdmle in order to compute
      max lik. of transformation parameters without unit i
      and procedure flr in order to compute the lik ratio test @

 i=1;    do until i==n+1;
    bsb=setdif(seq,i,1);
    {lamax}=fwdmle(X[bsb,.],seq1,0);
     lamax=lamax[.,2:nla+1];
     {pos,pot,lr,likmax,In,Un}=flr(X[bsb,.],seq1,ch,lamax,0);
      lrde[i,2]= lr[1,2];
    i=i+1;
  endo;

    if h==1;
       _pdate="";
     title("Deletion values of the Lik. ratio test");
       xy(lrde[.,1],lrde[.,2]);
    else;
    endif;

      retp(lrde);
    endp;

