/*
**  MEDB
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {P}=medb(X,d);                      To create a matrix which     79
**                                     contains bivariate
**                                     (univariate)
**                                     medians
**
*/

/*
**
** MEDB
**
** PURPOSE:  to create a matrix which contains bivariate L_1 medians
**           or univariate medians
**
** FORMAT:    {P}=medb(X,d);
**
** INPUT:     X     = N x p matrix of the data
**            d     = scalar, if d is equal to 1 bivariate L_1
**                    medians are calculated, else the program computes
**                    univariate medians.
**
**
** OUTPUT:    P     = p x p matrix which contains the coordinates
**                    of the bivariate (univariate) medians
**                    for each  couple
**                    of variables (the upper triangular part
**                    contains x coordinates, the lower triangular
**                    the y coordiantes)
**                    On the main diagonal matrix P contains zeros
**
** REMARK:  this program prepares a matrix which can be used
**          as input in program UNIBIV.G
**
** WARNING: two global variables named xglobal and yglobal are created
**
**
** EXAMPLE: if X= n x 3  and P =  0     1.3   1.1
**                                10.2   0    10.4
**                                2.5   2.1    0
**              when you consider the first and third variable
**              coordinate x of bivariate median =1.1
**              coordinate y  of bivariate median =2.5
*/


 clearg xglobal,yglobal;

    /* this procedure computes the univariate median */
proc (1)=univmed(e);
       local ran,c,em;
 ran=sortc(E,1);
 c=rows(ran);
 if (c/2-round(c-c/2))==0;
  Em=(ran[c/2]+ran[c/2+1])/2;
 else;
  Em=ran[(c+1)/2];
 endif;         @  Em  is the median  of vector E @
   retp(em);
endp;


/*******************************************************************/
/*    Procedure lf  computes the function to be minimized          */
/*    if you want to use the bivariate median                      */
/*******************************************************************/
proc (1)=lf(psi);
local mlogl;

  mlogl=sumc( abs(xglobal[.,1]-psi[1])+ abs(yglobal[.,1]-psi[2]) );

    retp(mlogL);
endp;

 proc (1)=medb(XX,d);
        local ii,iii,mlogl,psi0,psi,fvalue,grad,retcode,mP,p,
              umedx,umedy;
           p=cols(xx);
        mP=zeros(p,p);
  iii=1; do until iii==p;
    xglobal=xx[.,iii];
     ii=iii; do until ii==p;
       yglobal=xx[.,ii+1];
           if d==1;
                        psi0=meanc(xglobal)|meanc(yglobal);
                           {mlogl}=lf(psi0);
              {psi,fvalue,grad,retcode}=OPTMUM(&LF,psi0);
           else;
                          {umedx}=univmed(xglobal);
                          {umedy}=univmed(yglobal);
                            psi=umedx|umedy;
           endif;
    mP[iii,ii+1]=psi[1];
    mP[ii+1,iii]=psi[2];
                      ii=ii+1;
                   endo;
    iii=iii+1;
   endo;
   clear xglobal,yglobal;
  retp(mP);
 endp;

