/*
**  MULTSIMP
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {pos,pot}=multsimpl(X,bsb,euclid);
**
**                                This file contains a simplified version
**                                of multout to be used to construct
**                                the envelopes of min max and gap of MD 
**                                
*/

/*
** MULTSIMP
**
** PURPOSE:  Simplified version of procedure multout
**
** FORMAT:   {pos,pot}=multsimp(X,bsb,euclid);
**
** INPUT:     X = n x p data matrix; n observations
**                and p variables
**          bsb = m x 1 vector containing the units
**                belonging to the initial subset
**       euclid = scalar.
**                if euclid ==1; The progressing and the monitoring
**                   refers to Euclidean distances
**                else the progressing and the monitoring
**                   is based on Mahalanobis distances 
**
**
**
** OUTPUT:  pos = (n-m+1) x 6 matrix.
**                1st col = forward search index
**                2nd Col m-th ranked MD
**                3rd average of the first m ranked MD
**                4th average of the first m-1 MD
**                5th average of the MD of the unis forming BSB
**                6th max of MD among those forming BSB
**          pot = (n-m) x 3 matrix
**                1st col = forward search index
**                2nd col (m+1)-th ranked MD
**                3rd col min. Mahalanobis distances among the units
**                            which form the group of potential outliers
**
**    The output of the distances refers to Mahalanobis distances
**    only if input scalar euclid is different from 1
**    See also: MULTOUT.G
**
*/


 proc (2)=multsimp(X,bsb,euclid);

   local seq,ym,vc,malaf,ds,rdet,va,varX,tr,bs,out,
       nn,n,mm,i,gg,pos,pot,zs,min,mala,ii,oldbsb,ncl,sncl,sbsb;

nn=rows(X);
seq=SEQA(1,1,nn);
mm=rows(bsb);
min=mm;
bs=bsb;
@   The first column of pos contains the dimension
    of the sample used to compute Mahalanobis distances
    2nd Col m-th ranked MD
    3rd col max MD among those forming BSB  
    4th col Wilks ratio @
  pos=seqa(mm,1,nn-mm+1)~zeros(nn-mm+1,3);

@   pot has two columns the first contains the dimension of the
    sample used in order to compute Mahalanobis distances,
    2nd col (m+1)-th ranked MD
    3rd col min. Mahalanobis distances among the units
                 which form the group of potential outliers @
  pot=seqa(mm,1,nn-mm)~zeros(nn-mm,2);




/***  calculation of Mahal. dist. on initial clean data set ***/

mala=seq~zeros(nn,1);
@ Det. of var cov matrix based on all the data @
ds=det(vcx(X));

mm=mm; do until mm==nn+1;  @ nn= numero tot. di unita' stat. @
 @ "mm"; mm;  @
       ym=meanc(X[bsb,.]);
       va=vcx(X[bsb,.]);
@ 2nd col of Matrix Mala contains Mahalanobis distances _or Euclidean distances @

if euclid ==1;
             i=1;  do until i==nn+1;
                 mala[i,2]=sqrt((X[i,.]-ym')*(X[i,.]'-ym));
               i=i+1;
            endo;

else;
       vc=invpd(va);
             i=1;  do until i==nn+1;
                 mala[i,2]=sqrt((X[i,.]-ym')*vc*(X[i,.]'-ym));
               i=i+1;
            endo;
endif;


zs=sortc(mala,2);

               zs=zs|miss(zeros(10,2),0);
  @ sortc MD among those which belong to ncl @
  sbsb=sortc(mala[bsb,2],1);

                @   mm-th  max among bsb  _and wilks ratio @
 pos[mm-min+1,2:4]=zs[mm,2]~sbsb[mm]~(det(va)/ds);
            
     if mm<nn;

 
   ncl=setdif(seq,bsb,1);
 sncl=sortc(mala[ncl,2],1);

@ pot contains mm+1 th MD and  minimum of Mahalanobis distances among
  the units which form the group of potential outliers @
pot[mm-min+1,2:3]=zs[mm+1,2]~sncl[1];

@ the dimension of the clean data set increases of one unit.
  vector y contains the indexes corresponding to the units of
  the new basic subset @
       oldbsb=bsb;
      @  mm;  sortc(bsb,1)'; wait;  @
       zs=sortc(mala,2);

       bsb=zs[1:mm+1,1];

  
      else;
    endif;


                    mm=mm+1;
                  endo;
   retp(pos,pot);
endp;

