/*
**  NORM
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  {b1p,b2p}=norm(X)                   to test multivariate
**                                      normality
**
*/


/*
**
** NORM
**
** PURPOSE:  to test multivariate normality
**           see Mardia et al. 1979 pag. 21 and pag. 148
**
** FORMAT:   {b1p,b2p}=norm(X)
**
** INPUT:     X = n x p data matrix
**            d = scalar.
**                if d=1  program displays on the screen the values of
**                the tests of Skewness and Kurtosis
**                and the degres of freedom
**                else program computes tests
**                but nothing is printed on the screen
**
*/

/*
** OUTPUT:
**         b1p = multivariate measure of Skewness
**         b2p = multivariate measure of Kurtosis
**
*/


proc (2)=norm(X,d);
        local n,p,i,j,b1p,b2p,grs,m,S;
   n=rows(X);
   p=cols(X);
  b1p=0; @ b1p= multivariate measure of skewness @
  b2p=0; @ b2p= multivariate measure of kurtosis @

m=meanc(X);
S=invpd(vcx(X));
 i=1; do until i==n+1;
     j=1; do until j==n+1;

 grs=(X[i,.]-m')*S*(X[j,.]'-m);
      b1p=b1p+grs^3/(n^2);
                 if  i==j;
      b2p=b2p+grs^2/n;
            else;
        endif;
             j=j+1;
          endo;
      i=i+1;
  endo;    @ See also MArdia et al. 1970: p. 148 @
  b1p=(1/6)*n*b1p; @ this value must be compared with a \Chi^2_f
                    where f=(1/6)*p*(p+1)*(p+2);   @

  b2p=(b2p-p*(p+2))/sqrt(8*p*(p+2)/n);
                  @ this value must be compared with a N(0,1) @
      if d==1;
       "Multivariate skewness=";; b1p;
       "The coefficient of multivariate skewness must be compared";
       "with a \Chi^2 with ";; (1/6)*p*(p+1)*(p+2);; "degrees of freedom";

       "Multivariate kurtosis=";; b2p;
       "The coefficient of multivariate kurtosis must be compared";
       "with a N(0,1)";
      else;
      endif;
 retp(b1p,b2p);
endp;

