/*
**   OUTC.G
**   written by ALDO CORBELLINI & MARCO RIANI
**
**  FORMAT                         PURPOSE                     Line
** =======================================================================
** {out}=outc(X,Icentr,Spl)        To detect the units which lie
**                                 outside a (spline) curve
**
*/

/*
**
** OUTC
**
** PURPOSE:  To detect the units which lie
**           outside a (spline) curve
**
** FORMAT:    {out}=outc(X,Icentr,Spl)
**
** INPUT:     X = n x 3 matrix of data
**                1st col = x coordinates
**                2nd col = y coordiantes
**                3rd col = labels of the units
**       Icentr = 2 x 1 vector specifying the
**                coordinates of the centroid
**         Spl  = k x 2 matrix containing the
**                coordinates of the curve
**                1st col = x coordinates
**                2nd col = y coordiantes
**
** OUTPUT:  out = l x 3 matrix which contains the coordinates
**                of the units which lie outside the curve
**                1st col = x coordinates
**                2nd col = y coordiantes
**                3rd col = labels of the units
**
** REMARK: if no unit lies outside the curve out is a scAlar equal to 0
**
** See also: SPLINEM.G and BOXPLOTB.G
**
*/
proc (1)=outc(X,Icentr,Spl);
 local  nn,ll,mm,mx,my,xa,xb,xc,xd,xk,ya,yb,yc,yd,yk,eps,
             den,r,s,dp,dp1,dp2,dm,n,j,k,kk,wx,wy,xfin,y,ind;


  eps=1e-3;
if ismiss(x)==1;
    xfin=0;
    goto las;
else; 
  ind=X[.,3];
  y=X[.,2];
  X=X[.,1];
  xa = Icentr[1];
  ya = Icentr[2];
  wx=spl[.,1];
  wy=spl[.,2];
  Xfin=zeros(1,3);
 n=1; do while n<=rows(X);
      xb = x[n];
      yb = y[n];
   j = 1; do while j <rows(Spl);
      xc = wx[j];   xd = wx[j+1];
      yc = wy[j];   yd = wy[j+1];
      den = (xb - xa) * (yd - yc) - (yb - ya) * (xd - xc);
      r = ((ya - yc) * (xd - xc) - (xa - xc) * (yd - yc))/(den+1e-10);
      s = ((ya - yc) * (xb - xa) - (xa - xc) * (yb - ya))/(den+1e-10);
         if ((s >= 0 and s <= 1) and (r >= 0 and r <= 1));
              xk = xa + r * (xb - xa);
              yk = ya + r * (yb - ya);
              dp1 = sqrt((xk-xa)*(xk-xa)+(yk-ya)*(yk-ya));
              dp2 = sqrt((xb-xa)*(xb-xa)+(yb-ya)*(yb-ya));
              dp = sqrt((dp1-dp2)*(dp1-dp2));
                    if  dp > eps;
                        Xfin=Xfin|(xb~yb~ind[n]);
                       j=rows(Spl);
                    else;
                    endif;
         else;
         endif;
      j=j+1;
   endo;
  n=n+1;
 endo;

     if rows(xfin)==1;
       Xfin=0;
     else;
       Xfin=Xfin[2:rows(Xfin),.];
     endif;
endif;

las:

   retp(Xfin);
endp;

