/*
**  SCATTER
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**       scatter(X,AD);                   To create a scatter
**                                        plot matrix
**                                        It also produces
**                                        boxplots on the main diagonal
**                                        and different symbols for
**                                        the different groups
**
*/

/*
**
** SCATTER
**
** PURPOSE:  to create a scatter plot matrix
**
** FORMAT:    scatter(X,AD);
**
** INPUT:     X = n x p matrix of the data
**           AD = if AD = scalar
**                program assumes that all the units come from
**                the same universe (the same symbol is used for all
**                the units)
**                or vector g x 1 vector which contains the sizes of
**                the groups (REMARK: sumc(AD) must be equal to n)
**
**
** OUTPUT:     a scatter plot matrix on the screen with
**             univariate boxplots on the main diagonal
**
** Warning:    graphset is invoked
**             Univariate boxplots use gauss procedure box.g
**             
**
*/


  proc (0)=scatter(yyy,AD);
  local n,p,k,il,jl,mi,ma,step,x,y,S,sim,j,big,on,si,univ,
        g,og,max,size;

     graphset;
   n=rows(yyy);
      if rows(AD)>1 and sumc(AD) ne n;
       errorlog "The sum of the elements of vector AD is not equal to
                the number of rows of matrix X.";
        end;
      endif;

   p=cols(yyy); /**** # OF COLUMNS OF DATA MATRIX ****/
   S=6.855;  /***** DIMENSION OF THE SCREEN ****/
   k=1;

   g=rows(AD); @ g = number of groups @
   og=ones(g,1);
   if rows(AD)==1;
     AD=0|n;
   else;
     AD=0|AD;
   endif;
    
     max=maxc(ad);
     univ=miss(zeros(max,g),0);
                       on=ones(2,1);
                       big=10^29;
                       size=6.855;



    begwind;
        il=p-1;  DO UNTIL  il==-1; /*** il refers to the rows ****/
        jl=0;   DO UNTIL  jl==p;   /****** jl refers to the columns ****/

    @  rectangular plots @
    @ makewind(9/p,S/p,9/p*jl,S/p*il,1);  @
    @ square plots @
     makewind(6.855/p,6.855/p,6.855/p*jl,6.855/p*il,1);

       x=yyy[.,jl+1]; y=yyy[.,k];
           if jl+1==k;

         _psym=0;
         _PBOXCTL={0.2,2,2};
         _PLCTRL=1;
                   j=2; do while j<=rows(AD);
                        univ[1:ad[j],j-1]=
                        X[sumc(AD[1:j-1])+1:sumc(AD[1:j])];
                    j=j+1;
                   endo;
              box(0,univ);
               goto fin;


  @       _psym=0;
         _PBOXCTL={0.2,2,2};
         _PLCTRL=1;
              box(0,x);
               goto fin;   @
            else;
         _plctrl= -1;
         scale(x,y);

                    if rows(AD)>2;
                       si=  1|3|  4|  5|   6|7|12|13;
                       sim=zeros(1,7);
                       j=2; do while j<=rows(AD);
                         sim=sim|(X[sumc(AD[1:j-1])+1:sumc(AD[1:j])]~
                             y[sumc(AD[1:j-1])+1:sumc(AD[1:j])]~
                             (si[j-1])*ones(AD[j],1)~6*ones(AD[j],1)~
                             (j-1)*ones(AD[j],1)~ones(AD[j],1)~
                             9*zeros(AD[j],1));
                          j=j+1;
                       endo;
                       _psym=sim[2:rows(sim),.];
                       on=ones(2,1);
                       big=10^29;
                       xy(big*on,big*on);
                    else;
                      xy(x,y);
                    endif;
          endif;

              fin:
          jl=jl+1;
         nextwind;
          endo;
              k=k+1;
          il=il-1;
         endo;
   endwind;
  retp;
endp;

