/*
**  SCATTERB
**  written by MARCO RIANI
**
**  FORMAT                               PURPOSE              Line
** =======================================================================
**  scatterb(X,AD);                   To create a scatter       47
**                                    plot matrix with
**                                    superimposed bivariate
**                                    boxplots
**                                    The program also produces
**                                    univariate boxplots on the main
**                                    diagonal
**
*/

/*
**
** SCATTERB
**
** PURPOSE:  to create a scatter plot matrix with superimposed
**           bivariate boxplots. Univariate boxplots are also
**           produced. The program also copes with different
**           universes.
**
** FORMAT:    scatterb(X,AD);
**
** INPUT:     X = n x p matrix of the data
**           AD = if AD = scalar
**                program assumes that all the units come from
**                the same universe (the same symbol is used for all
**                the units)
**                or g x 1 vector which contains the sizes of
**                the groups (REMARK: sumc(AD) must be equal to n)
**
**
** OUTPUT:     a scatter plot matrix on the screen with
**             superimposed bivariate boxplots and univariate boxplots
**             on the main diagonal is produced
**
** Warning:    graphset is invoked
**             Univariate boxplots use gauss procedure box.g
**
**
*/


  proc (0)=scatterb(yyy,AD);
  local n,p,k,il,jl,mi,ma,g,step,x,y,S,sim,j,big,on,si,
        out,cent,Hul,Spl,X1,Y1,cen,og,max,univ,size,eps;

     graphset;
    n=rows(yyy);
      if rows(AD)>1 and sumc(AD) ne n;
       errorlog "The sum of the elements of vector AD is not equal to
                the number of rows of matrix X.";
        end;
      endif;

   p=cols(yyy); /**** # OF COLUMNS OF DATA MATRIX ****/
   S=6.855;  /***** DIMENSION OF THE SCREEN ****/
   k=1;
   g=rows(AD); @ g = number of groups @
   og=ones(g,1);
   if rows(AD)==1;
     AD=0|n;
   else;
     AD=0|AD;
   endif;
     max=maxc(ad);
     univ=miss(zeros(max,g),0);
                       on=ones(2,1);
                       big=10^29;
                       size=6.855;
    begwind;
        il=p-1;  do until  il==-1; @ il controls the rows @
        jl=0;   do until  jl==p;   @ jl controls the columns @

    @  rectangular plots @
    @ makewind(9/p,S/p,9/p*jl,S/p*il,1);  @

    @ square plots @
    makewind(size/p,size/p,size/p*jl,size/p*il,1);
       x=yyy[.,jl+1]; y=yyy[.,k];
           if jl+1==k;
         _psym=0;
         _PBOXCTL={0.2,2,2};
         _PLCTRL=1;
                   j=2; do while j<=rows(AD);
                        univ[1:ad[j],j-1]=
                        X[sumc(AD[1:j-1])+1:sumc(AD[1:j])];
                    j=j+1;
                   endo;
              box(0,univ);
               goto fin;
            else;
         _plctrl= -1;
         scale(x,y @ |vec(_pboxlim) @);
           _psym=0;
                       si=1|3|4|5|6|7|12|13;
                       sim=zeros(1,7);
                       cen=0~0;
                       j=2; do while j<=rows(AD);
                          X1=X[sumc(AD[1:j-1])+1:sumc(AD[1:j])];
                          Y1=y[sumc(AD[1:j-1])+1:sumc(AD[1:j])];
                         sim=sim|(X1~Y1~
                             (si[j-1])*ones(AD[j],1)~12*ones(AD[j],1)~
                             j*ones(AD[j],1)~ones(AD[j],1)~
                             9*zeros(AD[j],1));
                         {out,cent,Hul,Spl}=boxplotb(X1~Y1,1.58,0);
                          cen=cen|cent';
                          _plctrl=0;
                         @ Plot 50% spline @
                        xy(Spl[.,1],Spl[.,2]);
                         @ Plot outer spline @
                       @ xy(Spl[.,3],Spl[.,4]);  @
                          j=j+1;
                       endo;
                        @ cen= matrix which contains robust centroids
                          of the groups  @
                        cen=cen[2:g+1,.]~12*og~7*og~2*og~og~5*og;
                       _psym=sim[2:rows(sim),.]|cen;
                       xy(big*on,big*on);
          endif;

              fin:
          jl=jl+1;
         nextwind;
          endo;
              k=k+1;
          il=il-1;
         endo;

   endwind;
  retp;
endp;

