/*
**  SCODEL
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
**  {Scod}=scodel(X,y,la,h)             To compute deletion
**                                      diagnostics using
**                                      score test
**
*/


/*
**
** SCODEL
**
** PURPOSE:  to compute deletion diagnostics for
**           linear models using score test
**
**
**
** FORMAT:   {Scod}=scodel(X,y,la,h);
**
** INPUT:     X    = N x p matrix
**            y    = N x 1 response variable
**           la    = k x 1  vector which specifies for which values
**                   of the transformation parameter \lambda
**                   you want to compute the score test
**            h    = scalar dummy variable
**                   if d <> 1 no plot is produced
**                   if d=1 a plot of each column of matrix
**                   scodel  calculated
**                   (potential outlier is a unit which
**                    shows a sharp drop in the plot)
** OUTPUT:  Scod   = N x (nla+1) matrix
**                   nla = number of rows of vector lambda
**                   1st col. contains the indexes of the
**                   units,
**                   2nd col. deletion score test for la[1]
**                   3rd col. deletion score test for la[2]
**                             ...
**                   last col. deletion score test for la[nla]
**
** REMARK:   This method is very good when only one outlier is present
**           in the data, but it can suffer masking and swamping
**           problems when there are several outliers.
**           For more complicated methods based on
**           forward searches
*/


proc (1)=scodel(X,y,la,h);
      local seq,i,n,nla,bsb,Xd,yd,lik,tl,Scod,j;

  n=rows(X); nla=rows(la);
  seq=seqa(1,1,n);

 Scod=seq~miss(zeros(n,nla),0);
 i=1;    do until i==n+1;
    bsb=setdif(seq,i,1);
    Xd=X[bsb,.];
    yd=y[bsb];
        {tl,lik} = likla(Xd,yd,la);
      Scod[i,2:nla+1]= (tl');
    i=i+1;
  endo;

    if h==1;
       _pdate="";
     title("Deletion values of the score test");
      j=2; do until j==nla+2;
        @ xy(Scod[.,1],Scod[.,j]); @

      _pmsgctl=0.5~6.2~0.25~0.2~2~2~4;
      _pmsgstr= 0  $+"la="  $+ ftocv(la[j-1],4,2);
        iplot(Scod[.,j],0);
      j=j+1;
     endo;
    else;
    endif;

      retp(Scod);
    endp;

