/*
**   VARDEC.G
**   written by MARCO RIANI
**
**  FORMAT                      PURPOSE                     Line
** =======================================================================
** {W,B}=vardec(X,sz)        Decomposition of total          30
**                           deviance into groups and
**                           among groups
*/

/*
**
** VARDEC
**
** PURPOSE:  to  decompose a deviance matrix into the parts
**           due to differences within groups and among groups
**
** FORMAT:   {W,B}=vardec(X,sz)
**
** INPUT:    X = n x p data matrix
**          sz = g x 1 vector which contains the sizes of the groups
**
** OUTPUT:   W = p x p within groups deviance matrix
**           B = p x p between groups deviance matrix
**
*/


proc (2)=vardec(X,sz);
local n,g,W,i,T,B;
    n=rows(X);        @ n=rows del data set     @
     if sumc(sz) ne n;
      errorlog "The sum of the elements of vector sz is not equal to
                the number of rows of matrix X.";
       end;
     endif;

    g=rows(sz);       @ g=Number of groups @
    sz=0|sz;

    W=0; @ Initialize W @

    i=1; do until i==g+1;
       W= W+vcxm(X[(sumc(sz[1:i])+1):sumc(sz[1:i+1]),.])*sz[i+1];
     i=i+1;
    endo;

    T=vcxm(X)*n;
    B=T-W;
  retp(W,B);
endp;

