/*
**  WILKS
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {rt}=wilks(X,d);                   To compute deletion           47
**                                    diagnostics using ratio
**                                    among determinants
**
*/


/*
**
** WILKS
**
** PURPOSE:  to compute deletion diagnostics using
**           ratio among determinants.  The purpose is to create a
**           vector whose generic i-th  row contains |S_(i)|/|S|
**           where |S_(i)| is the determinant of sample covariance
**           matrix X excluding the i-th row of X and  |S|
**           is the determinant of the full sample covariance matrix
**
** FORMAT:   {rt}=wilks(X,d);
**
** INPUT:     X    = N x p matrix
**            d    = scalar dummy variable
**                   if d <> 1 no plot is produced
**                   if d=1 a plot of rt is calculated
**                   (potential outlier is a unit which
**                   shows a sharp drop in the plot)
** OUTPUT:    rt   = N x 2 matrix
**                   the first column contains the indexes of the
**                   units, the second column contains the ratio
**                   of determinants.
**
** REMARK:   This method is very good when only one outlier is present
**           in the data, but it can suffer masking and swamping
**           problems when there are several outliers.
**           For more complicated methods based on
**           forward searches see for example (Atkinson, 1994; JASA,
**           Atkinson and Riani, 1997; Environmetrics)
*/


 proc (1)=wilks(X,d);

      local seq,i,n,bsb,rt;
  n=rows(X);
  seq=seqa(1,1,n);
  rt=seq~zeros(n,1);

 i=1;    do until i==n+1;
    bsb=setdif(seq,i,1);
    rt[i,2]=det(vcx(X[bsb,.]));
    i=i+1;
  endo;
    rt[.,2]=rt[.,2]/det(vcx(X));

    if d==1;
       _pdate="";
       title("Deletion diagnostics based on ratio between determinants");
      iplot(rt[.,2],0);
    else;
    endif;

      retp(rt);
    endp;

