/*
**  WITHOUT.G
**
**  FORMAT                      PURPOSE                     Line
** =======================================================================
** {sa}=WITHOUT(m,n)          To sample without replacement
**
*/

/*
**
** WITHOUT
**
** PURPOSE:  To sample without replacement
**
** FORMAT:    {sa}=WITHOUT(m,n)
**
** INPUT:     m = scalar which specifies the
**                size of the sample
**            n = scalar specifying the number of units
**                from which to sample
**
** OUTPUT:    sa =  m x 1 vector containing the m units extracted
**
**            EXAMPLE  {sa}=without(5,20)
**                     extracts without replacement 5 units
**                     from a universe of size 20
**
** SEE ALSO  ELMS.G
**
*/

proc without(m,n);
  local s,i,x,sa;
  if m>n;
    "Error: cannot select m numbers from {1..n} if m>n";
    stop;
  endif;
  if ln(n) > (1.14*ln(m));
    s=ceil(seqa(n,-1,m).*rndu(m,1))+seqa(0,1,m);
    x=seqa(1,1,n);
    i=0; do while i<m; i=i+1;
      x[i s[i]]=x[s[i] i];
    endo;
  else;
    x=rankindx(rndu(n,1),1);
  endif;
    sa=x[1:m];
  retp(sa);
endp;

