/*
**  FWDBSB
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {BB,inter}=fwdbsb(X,bsb,forced,euclid);
**
**                                To return in each step of the
**                                fwd search, the units forming BSB
**
*/

/*
** FWDBSB
**
** PURPOSE:  to report the units forming bsb
**           in every step of the fwd procedure
**
** FORMAT:   {BB,inter}=fwdbsb(X,bsb,forced,euclid);
**
** INPUT:     X = n x p data matrix; n observations
**                and p variables
**          bsb = m x 1 vector containing the units
**                forming the initial subset
**       forced = scalar which controls the interchange
**                = 2 ==> no unit can leave the subset
**                = 1 ==> units forming initial bsb cannot leave
**                else ==> every unit can leave the subset
**       euclid = scalar.
**                if euclid ==1; The progressing and the monitoring
**                   refers to Euclidean distances
**                else the progressing and the monitoring
**                   is based on Mahalanobis distances 
**
**  OUTPUT   BB = n x (n-m+1) matrix. It reports for each
**                step of the forward procedure the indexes of the
**                units forming BSB
**        inter = (n-m) x 2 matrix
**                1st col = fwd search index
**                2nd col = interchanges in each step of the fwd search
**
**
**  See also: MULTOUT1, FWDSCREE,
**
*/


 proc (2)=fwdbsb(X,bsb,forced,euclid);

   local seq,ym,vc,nn,n,mm,i,gg,zs,min,mala,ii,BB,inter,oldbsb,bs,out;

nn=rows(X);
seq=SEQA(1,1,nn);
mm=rows(bsb);
min=mm;
bs=bsb;
@ BB = nn x nn-rows(f) is a matrix which reports for each step
of the forward procedure the indexes of the units reported in non
decreasing order of the Mahalanobis distance @
  BB=miss(zeros(nn,nn-mm),0);
@ inter =maxtrix which contains the number of interchanges in each step @
inter=seqa(mm+1,1,nn-mm)~miss(zeros(nn-min,1),0);

/***  calculation of Mahal. dist. on initial clean data set ***/
mala=seq~zeros(nn,1);

mm=mm; do until mm==nn+1;  @ nn= numero tot. di unita' stat. @

      ym=meanc(X[bsb,.]);
      

@ 2nd col of Matrix Mala contains Mahalanobis distances _or Euclidean distances @

if euclid ==1;
             i=1;  do until i==nn+1;
                 mala[i,2]=sqrt((X[i,.]-ym')*(X[i,.]'-ym));
               i=i+1;
            endo;

else;
              vc=invpd(vcx(X[bsb,.]));
                    i=1;  do until i==nn+1;
                 mala[i,2]=sqrt((X[i,.]-ym')*vc*(X[i,.]'-ym));
               i=i+1;
            endo;
endif;



      if rows(forced)==1 and forced==2;
          mala[bsb,2]=-ones(mm,1);
      elseif rows(forced)==1 and forced==1;
          mala[bs,2]=-ones(min,1);
      elseif rows(forced)==1 and forced==0;
      else;
          if mm<rows(forced);
            out=setdif(seq,forced,1);
            mala[out,2]=1e+16*ones(nn-rows(forced),1);
          else;
            mala[forced,2]=zeros(rows(forced),1);
          endif;
      endif;

   zs=sortc(mala,2);

    if mm<nn;
          oldbsb=bsb;
        @ the dimension of the clean data set increases of one unit.
          vector bsb contains the indexes corresponding to the units
          forming the new basic subset @
          bsb=zs[1:mm+1,1];
           BB[bsb,mm-min+1]=bsb;
          inter[mm-min+1,2]=rows(setdif(bsb,oldbsb,1));
     else;
     endif;

    mm=mm+1;
   endo;
 retp(BB,inter);
endp;

