/*
**  FWDELL
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {LB,PO}=fwdell(X,bsb,sel,st);
**
**                                To plot confidence ellipses
**                                in selected steps of the forward
**                                procedure
**
*/

/*
** FWDELL
**
** PURPOSE:   To produce scree plots of
**            ellipses in selected steps of the forward
**            procedure
**
** FORMAT:   {LB,PO}=fwdell(X,BB,sel,st)
**
** INPUT:     X = n x p data matrix; n observations
**                and p variables
**           BB = n x (n-m) matrix which contains the units
**                forming the subset in each step of the fwd search
**          sel = r x 1 vector which contains the selected steps
**                of the fwd procedure in which you want to see
**                the scree plot of Mahalanobis distances
**                if sel = scalar 0 then you can see all
**                elipses plots (in pairs of two) from m to n
**                REMARK: m \leq sel[j]  \leq n, j=1, ..., r
**           st = scalar which specifies the condifence levels
**                of the outer ellipses
**
**  OUTPUT   LB = rr x 4*rows(sel) matrix. It reports
**                for the steps specified in vector sel, the
**                coordinates of the ellipses
**                For example cols 1 and 2 report the x and y
**                coordinates to draw the inner ellipse in step sel[1]
**                cols 3 and 4 report the x and y coordiantes to
**                draw the outer ellipse in step sel[1]
**                .............
**                The last 4 colums report the
**                x and y coordinates to draw the two ellipses in step
**                sel[rows(sel)]
**           PO = n x 2*rows(sel) matrix. It reports for
**                for the steps specified by vector sel, the
**                coordinates of the points
**                For example cols 1 and 2 report the x and y
**                coordinates of the points in step sel[1]
**                .............
**                The last 2 colums report the
**                x and y coordinates of the points in step
**                sel[rows(sel)]
**
**
**   REMARK:  MATRIX BB CON BE CONSTRUCTED USING PROCEDURE FWDBSB
**
*/


 proc (2)=fwdell(X,BB,sel,st);

   local seq,nn,n,mm,y,xb,yb,Tx,Ty,Sx,Sy,r,Xs,Ys,E,Em,R1,R2,Th,
   Xx,Yy,The1,The2,Xmax,Ymax,emax,r1max,r2max,The1max,The2max,lin,
   linn,f,xalls,yalls,ran,bsb,j,mess1,LB,PO,pty,pcol;

  nn=rows(X);
  mm=rows(packr(BB[.,1]));
   @ Use procedure quelplot to draw an elipse @



 if sel==0;
  sel=seqa(1,1,n-mm);
 else;
  sel=sel-mm+1;
    if minc(sel)<=0;
       "Warning selected step smaller than initial subset dimension";
       "Program will stop";
             else;
       endif;
        
 endif;

  y=X[.,2];
  x=X[.,1];
  @ Matrix LB will contain the coordinates of the ellipses @
     LB=miss(zeros(360,rows(sel)*4),0);

  @ Matrix LB will contain the coordinates of the points @
     PO=miss(zeros(rows(x),rows(sel)*2),0);
j=1; do until j==rows(sel)+1;
      
      
     bsb=packr(BB[.,sel[j]]);
     n=rows(bsb);

      xb=X[bsb];
      yb=y[bsb];

      Tx=meanc(xb);
      Ty=meanc(yb);
      Sx=sqrt((xb-Tx)'(xb-Tx)/(n-1));    @ Sx is the  std of x @
      Sy=sqrt((yb-Ty)'(yb-Ty)/(n-1));   @ Sy is the  std of y @

       R=(xb-Tx)'(yb-Ty)/((n-1)*(Sx*Sy));   @ robust correlation @
       @ Standardize units forming the subset @
       Xs=(xb-Tx)/Sx;
       Ys=(yb-Ty)/Sy;
       @ Standardize all units using centroid of the subset @
       Xalls=(x-Tx)/Sx;
       Yalls=(y-Ty)/Sy;


    /* Now we calculate Mahalanobis distances from
       centroid previously defined */
       E=sqrt((Xs.^2+Ys.^2-2*R*Xs.*Ys)/(1-R^2));

      /** the 50th percentile of an F with 2 and rows(x)-2 d.o.f.***/
       ran=sortc(E,1);
        if (n/2-round(n-n/2))==0;
              Em=(ran[n/2]+ran[n/2+1])/2;
        else;
              Em=ran[(n+1)/2];
        endif;         @  Em  is the median  of vector E @

       /** the square of the MAha. distance is distributed like an F **/
       @ n number of rows of x (y) @
       eMAX= (2*(n-1))/(n-2)* st;
       @ d = constant which specifies the condidence level of the outer
            ellipse @
       emax=sqrt(emax);
       emax=ran[rows(ran)];

        /*** code for the hinge  ***/
        R1=Em*sqrt((1+R)/2);
        R2=Em*sqrt((1-R)/2);
        th=seqa(0,2*pi/360,360);
        The1=R1*cos(th);
        The2=R2*sin(th);
        Xx=(The1+The2);
        Yy=(The1-The2);

        /*** code for the fence ***/
        R1max=Emax*sqrt((1+R)/2);
        R2max=Emax*sqrt((1-R)/2);
        The1max=R1max*cos(th);
        The2max=R2max*sin(th);
        Xmax=(The1max+The2max);
        Ymax=(The1max-The2max);

    @    Xx=(Xx-Tx)/Sx;
        Yy=(Yy-Ty)/Sy;
        Xmax=(Xmax-Tx)/Sx;
        Ymax=(Ymax-Ty)/Sy;
     @
        scale(Xmax|Xalls,Ymax|Yalls);
           begwind;
           window(1,1,0);
           makewind(80,6.8555,0,0,1);


       /**** plot of the hinge together with the fence ***/
        lin=1~6~0~-100~0~100~0~7~0.2;
        linn=1~6~-100~0~100~0~0~7~0.2;
        _pline=lin|linn;
     @ plot the size of the subset @

     mess1= 0 $+ "m=" $+ ftocv(sel[j]+mm-1,2,0);
   _PMSGCTL=1~2~0.3~0~1~2;
   _PMSGSTR =mess1;
   pty=ones(nn,1);
   "bsb"
   bsb;
   "dnjhdkljkl";
   rows(bsb);
   n;
   rows(pty);
   
   pty[bsb]=3*ones(n,1);
 @  PTY[36]=8;  @
   pcol=6*ones(nn,1);
   pcol[bsb]=3*ones(n,1);
         @ BSB'; WAIT; @
      _pltype=6|6;
      @ _psym indicates the location of the points in the main plot @
      _psym=Xalls~Yalls~pty~3*ones(nn,1)~pcol~
      1.5*ONES(nn,1)~2*ONES(nn,1);
      _pnumht=0.10|0.10;
         @   xtics(-1,1,0.1,1);
            ytics(-12,12,4,1);   @

      xy(XX~xmax,YY~ymax);
      endwind;


      LB[.,(4*j-3):j*4]=XX~YY~xmax~ymax;
      PO[.,(2*j-1):j*2]=Xalls~Yalls;
    j=j+1;
   endo;
   retp(LB,PO);
endp;

