/*
**  FWDMLES
**  written by MARCO RIANI
**
**  FORMAT                              PURPOSE                     Line
** =======================================================================
** {mL}=fwdmleS(X,bsb,la);             To create a matrix which
**                                     contains MLE of transformation
**                                     parameters in each step
**                                     of the forward search
**                                     NOTE THAT WE SEARCH FOR
**                                     A COMMON OF LAMBDA FOR ALL THE
**                                     VARIABLES
**
*/

/*
**
** FWDMLE
**
** PURPOSE:  to create a vector wchih contains Max. Lik. Est.
**           of common transformation parameter at each step of
**           the forward search
**
**
** FORMAT:    {mL}=fwdmleS(X,bsb,la);
**
** INPUT:     X     = n x p matrix of the data
**           bsb    = m x 1 vector which contains the indexes of the
**                    units forming the initial subset
**            la    = scalar which specifies  which value of
**                    lambda must be used for ordering the units
**                    in all steps of the forward search
**
** OUTPUT:    mL    = (n-m+1)  x 2 matrix. In the first column
**                    it contains the dimension of the basic subset used
**                    in the forward search. In column 2 we
**                    can find max. lik. estimate of the common
**                    transformation
**                    parameter in each step of the forward search
**
** REMARK:  the input vector bsb can be conveniently created by
**          program UNIBIV.G
**          The transformtion used in the maximization is 2.5*sin(\lambda)
**          The initial \lambda values are =1 (no transformation)
**
**
** See also: FLR,FWDMLER,FWDMLEQD,FWDMLELD,MULTOUT,FWDMLE
**
** WARNING: one global variable named Xglobal is created
**
**
*/

 clearg Xglobal;


proc (1)=assS(la0);    /* procedure which transforms the parameters */
     local la;
                        @  la=  5*(sin  (  la0  ));   @
                        @   la=3*la0/sqrt(1+la0^2);   @
                        @  la= 5*( 1-exp(la0))/(1+exp(la0)) ; @
                          la=  la0;
    retp(la);
endp;


proc (1)=likS(la);   /* procedure which computes max. lik. */
     local G,Z,dZ;
             Z=Xglobal;
             {la}=assS(la);
                  G=exp(meanc(ln(Xglobal)));
                   G=G'; @ Vector of geometric means @
                   @   just Z[.,chglobal] is tested @
             Z=(Xglobal^(la)-1
                              )./((la).*G.^((la')-1));
       dZ=100*det(vcx(Z));

   retp(dZ);
endp;

proc (1)=fwdmleS(X,bsb,la); /* procedure which computes fwd search */
       local nn,seq,Plik,mm,la0,zs,mala,min,dZ,Xt,sel,j,
             psi,fvalue,grad,retcode,vc,i,ym,psin,p,prov,G;
          nn=rows(X);
          p=cols(X);
          seq=SEQA(1,1,nn);
          mm= rows(bsb);
          min=mm;

          Plik=seqa(mm,1,nn-mm+1)~ones(nn-mm+1,1);

          /*  calculation of Mahal. dist. on initial clean data set */
           mala=seq~zeros(nn,1);
           la0=la;
          @ Create Xt=matrix of transformed values to be used
             in the forward search (for ordering MD) @
          Xt=X;

                  G=exp(meanc(ln(vec(Xt))));
           if la NE 0;
                  Xt= X^la./(G^(la-1));
              else;
                  Xt=ln(X);
             endif;

       mm=mm; do until mm==nn+1;  @ nn= numero tot. di unita' stat. @
                   if mm>min;
                  la0=psi;
               else;
               endif;

                  Xglobal=X[bsb,.];
                  {dZ}=likS(la0);

                  {psi,fvalue,grad,retcode}=OPTMUM(&likS,la0);
                   @ Retransform the MLE parameters @
                            {psin}=assS(psi);
                    Plik[mm-min+1,2]=psin;

               ym=meanc(Xt[bsb,.]);
               vc=invpd(vcx(Xt[bsb,.]));

                            i=1;  do until i==nn+1;
                             mala[i,2]=sqrt((Xt[i,.]-ym')*vc*(Xt[i,.]'-ym));
                                 i=i+1;
                             endo;
                zs=sortc(mala,2);

                        @ the dimension of the clean data set increases
                          by one unit. Vector bsb contains the
                          indexes corresponding to the units of
                          the new basic subset @
                                if mm<nn;
                                   bsb=zs[1:mm+1,1];
                                 else;
                                 endif;
             mm=mm+1;
          endo;
  retp(Plik);
endp;

