/*
**   SPLINEM.G
**   written by ALDO CORBELLINI & MARCO RIANI
**
**  FORMAT                         PURPOSE                         Line
** =======================================================================
** {SPL}=splineM(z)                 To superimpose a B-spline curve 	36
**                                  on a polygon
**
*/

/*
**
** PURPOSE: To superimpose a B spline curve over a
**          polygon
**
** FORMAT:   {SPL}=splineM(z)
**
** INPUT:     z = m x k matrix of data
**                (only the first two colums are used)
**                containing the coordinates of the m points
**                which form the polygon
**                1st col = x coordinates
**                2nd col = y coordiantes
**
** OUTPUT:  SPL = k x 2 matrix which contains the coordinates
**                of the spline curve
**                1st col = x coordinates
**                2nd col = y coordiantes
**
** See also: HULL
**
*/


proc (1)=splineM(zeta);
  local A,a2,x,L,sei,due,res,N,i,j,X1,t,tv,b2,
        b3,Spl;

       A={ 1 -3  3 -1,
           4  0 -6  3,
           1  3  3 -3,
           0  0  0  1 };

       x=zeta|zeta[1:minc(5|rows(zeta)),.];
       L=rows(X);

       sei=1/6; due=0.5;
       res=50;  @ risoluzione della spline punti per segmento @
       N=res; tv=1;

       i=2;
       j=1;
       b3=0|0;

       do while i < L-1;
           X1=X[i-1:i+2,1:2]';
           a2=sei*X1*A;
             j = 1;
              do while j < res;
                  t=j/N;
                  tv=1|t|t^2|t^3;
                  b2=a2*tv;
                  j=j+1;
                  b3=b3~b2;
              endo;
           i=i+1;
       endo;
        @  cols(b3);
          (res-1)*(L-4);  wait;  @
          Spl=b3[.,2:(res-1)*(L-4)+10]';
 retp(Spl);
endp;

